//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright   "Money-Meter (by Transcendreamer)"
#property description "Chart evaluation in deposit currency"
#property strict
#property indicator_chart_window
#property indicator_plots 0

//+------------------------------------------------------------------+
#include <Trade\PositionInfo.mqh>
#include <Trade\Trade.mqh>
#include <Trade\SymbolInfo.mqh>

CPositionInfo  m_position;                   // trade position object
CTrade         m_trade;                      // trading object
CSymbolInfo    m_symbol;                     // symbol info object
CDealInfo      m_deal;                       // deals object
COrderInfo     m_order;                      // pending orders object

//+------------------------------------------------------------------+

input double            zero_price        = 0; // 0:Current/xx:Starting-Price
input double            lot_size          = 0.01;
input double            grid_step_value   = 100;

input double            lot_divider       = 1;
input double            total_levels      = 10;
enum  PROGRESSION       {none,equal,linear,fibo,martin};
input PROGRESSION       progression=none;
input double            multiplicator     = 2;
input color             lines_color       = clrPink;
input int               lines_width       = 1;
input ENUM_LINE_STYLE   lines_style       = STYLE_DOT; 
    //STYLE_SOLID
input bool              lines_prices      = false;
input int               text_shift_bars   = 0;
input ENUM_BASE_CORNER  info_corner       = CORNER_LEFT_LOWER;
input int               info_shift_pixels = 0;
input bool              Show_Grid         = true;
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnInit()
  {
   clean_all();
   make_grid();
   //put_info();
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   clean_all();
   Comment(" ");
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   make_grid();
   return(rates_total);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void clean_all()
  {
   ObjectDelete(0,"INFOTEXT");
   ObjectDelete(0,"GRID_*");
   
   for(int k=ObjectsTotal(0,0,OBJ_HLINE)-1; k>=0; k--)
     {
      string name=ObjectName(0,k,0,OBJ_HLINE);
      if(StringFind(name,"GRID_LEVEL_")!=-1)
         ObjectDelete(0,name);
     }
   for(int k=ObjectsTotal(0,0,OBJ_TEXT)-1; k>=0; k--)
     {
      string name=ObjectName(0,k,0,OBJ_TEXT);
      if(StringFind(name,"GRID_TEXT_")!=-1)
         ObjectDelete(0,name);
     }
  }

  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double get_progression(int n)
  {
   if(progression==none) return(n);
   double sum=0;
   for(int k=1; k<=n; k++)
      sum+=(n-k+1)*get_member(k);
   return(sum);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double get_member(int k)
  {
   if(progression==equal)
     {
      return(1);
     }
   else if(progression==fibo)
     {
      if(k<3) return(1);
      int f=1,s=1;
      while(k>2) { f=f+s*2; s=f-s; f=f-s; k--; }
      return(s);
     }
   else if(progression==martin)
     {
      return(MathPow(multiplicator,k-1));
     }
   else if(progression==linear)
     {
      return(k);
     }
   return(1);
  }
  
/*
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void put_info()
  {
   ObjectCreate(0,"INFOTEXT",OBJ_LABEL,0,0,0);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_XDISTANCE,5);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_YDISTANCE,17+info_shift_pixels);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_SELECTED,false);
//---
   ENUM_ANCHOR_POINT anchor=0;
   if(info_corner==CORNER_LEFT_LOWER)  anchor=ANCHOR_LEFT_LOWER;
   if(info_corner==CORNER_LEFT_UPPER)  anchor=ANCHOR_LEFT_UPPER;
   if(info_corner==CORNER_RIGHT_LOWER) anchor=ANCHOR_RIGHT_LOWER;
   if(info_corner==CORNER_RIGHT_UPPER) anchor=ANCHOR_RIGHT_UPPER;
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_CORNER,info_corner);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_ANCHOR,anchor);
//---
   //string text="LOT:"+DoubleToString(lot_size/lot_divider,2);
   if(progression==none)   text+=("  MODE:NONE");
   if(progression==equal)  text+=("  MODE:EQUAL");
   if(progression==linear) text+=("  MODE:LINEAR");
   if(progression==fibo)   text+=("  MODE:FIBO");
   if(progression==martin) text+=("  MODE:MARTIN *"+DoubleToString(multiplicator,2));
   ObjectSetString(0,"INFOTEXT",OBJPROP_TEXT,text);
   ObjectSetString(0,"INFOTEXT",OBJPROP_FONT,"Verdana");
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_COLOR,lines_color);
  }
*/

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void make_grid()
  {
//--

   int      total_positions = 0;
   double   total_profit    = 0.0;
   int      count_buys      = 0,count_sells   = 0;
   double   tot_vol_buy     = 0, tot_vol_sell = 0,av_prc_buy =0;
   double   profit_buy      = 0, profit_sell  = 0,av_prc_sell=0;
   double   volume_biggest_buys  = 0.0, volume_biggest_sells = 0.0;

   CalculatePositionsAndProfit(total_positions,total_profit,
                               profit_buy, profit_sell,
                               count_buys,count_sells, 
                               tot_vol_buy,tot_vol_sell,
                               av_prc_buy,av_prc_sell,
                               volume_biggest_buys,volume_biggest_sells
                               );

  double z_prc,z_lot,z_lot_div,zero;

// no open order
  if ((av_prc_buy==0) && (av_prc_sell==0))
      {
      z_prc = zero_price;
      z_lot = lot_size;
      }
//--
  
  if ((av_prc_buy>0) && (av_prc_sell>0)) // have SELL & BUY orders
      {
      z_prc = ((av_prc_buy+av_prc_sell)/2);
      if (count_buys>count_sells) double z_lot = (tot_vol_buy-tot_vol_sell);
      if (count_buys<count_sells) double z_lot = (tot_vol_sell-tot_vol_buy);
      }
//--

  if ((av_prc_buy>0) && (av_prc_sell==0)) // only have BUY orders
      {
      z_prc = av_prc_buy;
      z_lot = tot_vol_buy;
      }
//--

  if ((av_prc_buy==0) && (av_prc_sell>0)) // only have SELL orders
      {
      z_prc = av_prc_sell;
      z_lot = tot_vol_sell;
      }


      z_lot_div = lot_divider ; //AccountInfoInteger(ACCOUNT_LEVERAGE);
      if (z_prc>0)  {zero = z_prc;}
        else if (z_prc==0) {zero = iClose(NULL,0,0);}
      
      string text="LOT:"+DoubleToString(z_lot/z_lot_div,2);      
//--  
   double ts=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE);
   double tv=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE);
   //double step=grid_step_value/tv*ts/lot_size/lot_divider;
   //double zero=zero_price?zero_price:iClose(NULL,0,0);
   
   double step=grid_step_value/tv*ts/z_lot/z_lot_div;

//--   
   ObjectCreate(0,"INFOTEXT",OBJ_LABEL,0,0,0);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_XDISTANCE,5);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_YDISTANCE,17+info_shift_pixels);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_SELECTED,false);
//---
   ENUM_ANCHOR_POINT anchor=0;
   if(info_corner==CORNER_LEFT_LOWER)  anchor=ANCHOR_LEFT_LOWER;
   if(info_corner==CORNER_LEFT_UPPER)  anchor=ANCHOR_LEFT_UPPER;
   if(info_corner==CORNER_RIGHT_LOWER) anchor=ANCHOR_RIGHT_LOWER;
   if(info_corner==CORNER_RIGHT_UPPER) anchor=ANCHOR_RIGHT_UPPER;
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_CORNER,info_corner);
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_ANCHOR,anchor);
//---
   text="LOT:"+DoubleToString(z_lot/z_lot_div,2)+" Av.Price : "+ DoubleToString(z_prc,_Digits);
   if(progression==none)   text+=("  MODE:NONE");
   if(progression==equal)  text+=("  MODE:EQUAL");
   if(progression==linear) text+=("  MODE:LINEAR");
   if(progression==fibo)   text+=("  MODE:FIBO");
   if(progression==martin) text+=("  MODE:MARTIN *"+DoubleToString(multiplicator,2));
   ObjectSetString(0,"INFOTEXT",OBJPROP_TEXT,text);
   ObjectSetString(0,"INFOTEXT",OBJPROP_FONT,"Verdana");
   ObjectSetInteger(0,"INFOTEXT",OBJPROP_COLOR,lines_color);   
//--   
 /*  
    Comment("\n"+"\n"
         + " ts    : " + string(ts)+"\n"
         + " tv    : " + string(tv)+"\n"
         + " S_cnt : " + string(count_sells)+"\n"
         + " B_cnt : " + string(count_buys)+"\n"
         + " z_prc : " + string(z_prc)+"\n"
         + " z_lot : " + string(z_lot)+"\n"    
         + " step  : " + string(step)
           );
 */           
//---
   datetime time;
   if(text_shift_bars>=0)
      time=iTime(NULL,0,text_shift_bars);
   else
      time=iTime(NULL,0,0)-PeriodSeconds(PERIOD_CURRENT)*text_shift_bars;
//---
   put_level("GRID_LEVEL_ZERO",zero,"(START)");
    put_text("GRID_TEXT_ZERO",zero,time,"(START)");
   
  if (Show_Grid == true)
  {
   for(int n=1; n<=total_levels; n++)
     {
      double value=grid_step_value*get_progression(n);
      string text=DoubleToString(value,2);
      put_level("GRID_LEVEL_UP"+string(n),zero+step*n,"+"+text);
      put_level("GRID_LEVEL_DN"+string(n),zero-step*n,"-"+text);
      put_text("GRID_TEXT_UP"+string(n),zero+step*n,time,"+"+text);
      put_text("GRID_TEXT_DN"+string(n),zero-step*n,time,"-"+text);
     }
  }   
  
  }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void put_level(string name,double price,string text)
  {
   ObjectCreate(0,name,OBJ_HLINE,0,0,0);
   ObjectSetDouble(0,name,OBJPROP_PRICE,price);
   ObjectSetInteger(0,name,OBJPROP_COLOR,lines_color);
   ObjectSetInteger(0,name,OBJPROP_WIDTH,lines_width);
   ObjectSetInteger(0,name,OBJPROP_STYLE,lines_style);
   ObjectSetInteger(0,name,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,name,OBJPROP_SELECTED,false);
   ObjectSetInteger(0,name,OBJPROP_BACK,!lines_prices);
   ObjectSetString(0,name,OBJPROP_TEXT,text);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void put_text(string name,double price,datetime time,string text)
  {
   ObjectCreate(0,name,OBJ_TEXT,0,0,0);
   ObjectSetDouble(0,name,OBJPROP_PRICE,price);
   ObjectSetInteger(0,name,OBJPROP_TIME,0,time);
   ObjectSetString(0,name,OBJPROP_TEXT,text);
   ObjectSetString(0,name,OBJPROP_FONT,"Verdana");
   ObjectSetInteger(0,name,OBJPROP_COLOR,lines_color);
   ObjectSetInteger(0,name,OBJPROP_FONTSIZE,8);
   ObjectSetInteger(0,name,OBJPROP_ANCHOR,ANCHOR_RIGHT_UPPER);
   ObjectSetInteger(0,name,OBJPROP_SELECTABLE,false);
   ObjectSetInteger(0,name,OBJPROP_SELECTED,false);
  }
//+------------------------------------------------------------------+

void CalculatePositionsAndProfit(int &total_positions,double &total_profit,
                                 double &profit_buy,double &profit_sell, 
                                 int &count_buys,int &count_sells,
                                 double &tot_vol_buy,double &tot_vol_sell,
                                 double &av_prc_buy,double &av_prc_sell,
                                 double &volume_biggest_buys,double &volume_biggest_sells
                                )
  {
   total_positions = 0;
   total_profit    = 0.0;
   count_buys      = 0;   count_sells     = 0;   
   tot_vol_buy     = 0;   tot_vol_sell    = 0;   
   profit_buy      = 0;   profit_sell     = 0;
   av_prc_buy      = 0;   av_prc_sell     = 0;
   volume_biggest_buys  = 0.0; volume_biggest_sells = 0.0;
   
   for(int i=PositionsTotal()-1;i>=0;i--)
     {
      if(m_position.SelectByIndex(i)) // selects the position by index for further access to its properties
        {
         if(m_position.Symbol()==Symbol()) // && m_position.Magic()==m_magic)
          {
         
         total_positions++; // open oreder count; but not volume
         total_profit=total_profit+m_position.Commission()+m_position.Swap()+m_position.Profit();
         
            if(m_position.PositionType()==0) //POSITION_TYPE_BUY)
              {
               count_buys++; tot_vol_buy = tot_vol_buy + m_position.Volume();
               profit_buy = profit_buy + m_position.Commission()+m_position.Swap()+m_position.Profit();
               av_prc_buy = av_prc_buy + m_position.PriceOpen()*m_position.Volume();
               if(m_position.Volume()>volume_biggest_sells) {volume_biggest_sells=m_position.Volume();}
              }
            if(m_position.PositionType()==1) //POSITION_TYPE_SELL)
              {
               count_sells++; tot_vol_sell = tot_vol_sell + m_position.Volume();
               profit_sell = profit_sell + m_position.Commission()+m_position.Swap()+m_position.Profit();
               av_prc_sell = av_prc_sell + m_position.PriceOpen()*m_position.Volume();
               if(m_position.Volume()>volume_biggest_sells) {volume_biggest_sells=m_position.Volume();}               
              }         

          }

        }
     }   
     if (count_buys !=0) {av_prc_buy  = (av_prc_buy/tot_vol_buy);}
     if (count_sells!=0) {av_prc_sell = (av_prc_sell/tot_vol_sell);}
//---
   return;
  }
//---------------------------------------------------------------------------------------

